/********************************************************************
*  Licensed Materials - Property of IBM
*  
*  (c) Copyright IBM Corp.  2006, 2007 All Rights Reserved
*  
*  US Government Users Restricted Rights - Use, duplication or
*  disclosure restricted by GSA ADP Schedule Contract with
*  IBM Corp.
********************************************************************/

package examples.api.recon;

import java.rmi.*;
import java.util.*;

import com.ibm.itim.apps.*;
import com.ibm.itim.apps.exception.AppProcessingException;
import com.ibm.itim.apps.provisioning.*;
import com.ibm.itim.apps.recon.*;
import com.ibm.itim.apps.search.*;
import com.ibm.itim.dataservices.model.*;
import com.ibm.itim.dataservices.model.domain.*;
import com.ibm.itim.remoteservices.exception.RemoteServicesException;

import examples.api.recon.callback.ReconCompleteCallback;
import examples.api.Utils;

import javax.security.auth.*;
import javax.security.auth.login.*;

/**
 * Sample command-line Java class to run recon. This example adds
 * a new recon unit and runs reconciliation with this unit. 
 *
 * To run a recon using existing reconciliation unit -
 * 1. Modify program to accept only schedule information of an existing
 *    reconciliation unit.
 * 2. Use 'ReconManager.getReconUnits(serviceMO);' to get all reconciliation
 *    units, and identify the one having the same schedule as specified
 *    as input.
 * 3. Use this reconciliation unit or ReconUnitData object as a parameter
 *    to one of the 'runRecon' functions of class - 
 *    com.ibm.itim.apps.recon.ReconManager.
**/
public class RunRecon { 

   /**
    * Command line argument names (prefixed by "-")
    */
   private static final String SERVICE_PROFILE    = "serviceprofile";
   private static final String SERVICE_FILTER     = "servicefilter";
   private static final String RECON_FILTER       = "reconfilter";
   private static final String EXCLUDE_ATTRIBUTES = "excludeattributes";
   private static final String LOCK_SERVICE       = "lockservice";
   private static final String MAX_DURATION       = "maxduration";
   private static final String RECON_CALLBACK     = "reconcallback";
   private static final String MONTH              = "month";
   private static final String DAY                = "day";
   private static final String DAY_OF_WEEK        = "dayofweek";
   private static final String HOUR               = "hour";
   private static final String MINUTE             = "minute";
   private static final String NAME               = "name";
   private static final String DESCRIPTION        = "description";

   private static final String[][] utilParams = new String[0][0];

   /**
    * main method.
    */
   public static void main(String[] args) {

        Utils utils = new Utils(utilParams, true);
        Hashtable arguments = parseArgs(args);
        PlatformContext platform = null;

        try {
           String tenantId = utils.getProperty(Utils.TENANT_ID);
           String ldapServerRoot = utils.getProperty(Utils.LDAP_SERVER_ROOT);

           platform = utils.getPlatformContext();
           Subject subject = utils.getSubject(platform);

           System.out.println("Searching for Service \n");
           // Use the Search API to locate the the Service instance
           SearchMO searchMO = new SearchMO(platform, subject);
           searchMO.setContext(new CompoundDN(new 
                     DistinguishedName("ou="+tenantId+"," + ldapServerRoot)));
           searchMO.setCategory(ObjectProfileCategory.SERVICE);
           searchMO.setFilter((String)arguments.get(SERVICE_FILTER));
           searchMO.setProfileName((String)arguments.get(SERVICE_PROFILE));
           SearchResultsMO searchResultsMO = null;
           Collection services = null;
            try {
                searchResultsMO = searchMO.execute();
                services = searchResultsMO.getResults();
                if(services == null || services.isEmpty()) {

                    System.out.println("Unable to find Service.\n");
                    System.exit(0);
                }
            } finally {
                // close SearchResultsMO
                if(searchResultsMO != null) {
                    try {
                        searchResultsMO.close();
                    } catch(Exception e) {
                        e.printStackTrace();
                    }
                }
            }

           Service service = (Service)services.iterator().next();
           ServiceMO serviceMO =
              new ServiceMO(platform, subject, service.getDistinguishedName());

           System.out.println("Creating ReconManager \n");
           ReconManager reconManager = new ReconManager(platform, subject);

           
           // Initialize temporary reconciliation unit
           ReconUnitData newReconUnitData = new ReconUnitData();

           // Set Name
           String name = (String)arguments.get(NAME);
           if (name == null || name.trim().length() == 0) {
               System.out.println("Invalid name specified\n");
               System.exit(0);
           }
           newReconUnitData.setName(name);

           // Set Description
           String description = (String)arguments.get(DESCRIPTION);
           if (description != null && description.trim().length() > 0) {
               newReconUnitData.setDescription(description);
           }

           // Set month
           int month = Integer.parseInt((String)arguments.get(MONTH));
           if(month < -1 || month > 12) {
               System.out.println("Invalid month specified. " + 
                   "Valid values : 1-12, -1 stands for Monthly recon and 0 to ignore.\n");
               System.exit(0);
           }
           newReconUnitData.setMonth(month);

           // Set day of month
           if((String)arguments.get(DAY) != null) {
               int day = Integer.parseInt((String)arguments.get(DAY));
               if(day < -1 || day > 31) {
                   System.out.println("Invalid day specified. " + 
                       "Valid values : 1-31, -1 stands for Daily recon and 0 to ignore.\n");
                   System.exit(0);
               }
               newReconUnitData.setDayOfMonth(day);
           }

            //Set day of week
            //Should be set only if the day of month is not required
            if((String)arguments.get(DAY) == null && (String)arguments.get(DAY_OF_WEEK) != null) {
                int dayOfWeek = Integer.parseInt((String)arguments.get(DAY_OF_WEEK));
                if(dayOfWeek < 0 || dayOfWeek > 7) {
                    System.out.println("Invalid day of week specified. " + 
                        "Valid values 1-7 (1 = Sunday, 2 = Monday, ...) and 0 to ignore.\n");
                    System.exit(0);
                }
                newReconUnitData.setDayOfWeek(dayOfWeek);
            }

           // Set hour
           int hour  = Integer.parseInt((String)arguments.get(HOUR));
           if(hour < -1 || hour > 23) {
               System.out.println("Invalid hour specified. " + 
                   "Valid values : 0-23 and -1 stands for Hourly recon.\n");
               System.exit(0);
           }            
           newReconUnitData.setHour(hour);

           // Set minute
           int minute = Integer.parseInt((String)arguments.get(MINUTE));
           if(minute < 0 || minute > 59) {
               System.out.println("Invalid minute specified. " + 
                   "Valid values : 0-59.\n");
               System.exit(0);
           }
           newReconUnitData.setMinute(minute);

           // Set max duration of reconciliation
           String strMaxDuration = (String)arguments.get(MAX_DURATION);
           if(strMaxDuration != null && strMaxDuration.trim().length() > 0) {
            newReconUnitData.setMaxDuration(Long.valueOf(strMaxDuration).longValue());
           }

           // Set lock-service setting
           String lockService = (String)arguments.get(LOCK_SERVICE);
           if(lockService != null) {
               newReconUnitData.setLockService(Boolean.valueOf(lockService).booleanValue());
           }

           // Set reconciliation filter if supported
           String reconQueryFilter  = (String) arguments.get(RECON_FILTER);
           if(reconManager.isFilterSupported(serviceMO)) {
               newReconUnitData.setReconQueryFilter(reconQueryFilter);
           } else {
               System.out.println("Recon query filter not supported, query is ignored.\n");
           }

           // Set reconciliation attributes to be included.
           Collection attributes = reconManager.
                         getAvailableReconciliationAttributes(serviceMO);
           String excludeAttributes = (String) arguments.get(EXCLUDE_ATTRIBUTES);
           if(excludeAttributes != null) {

               String[] excludeAttribList = excludeAttributes.split(",");
               ArrayList<String> excludeQueryAttribures = new ArrayList<String>();
               for(int i=0;i<excludeAttribList.length; i++) {
                  excludeQueryAttribures.add(excludeAttribList[i]);
               }
               newReconUnitData.setReconQueryAttributes(attributes, 
                                         excludeQueryAttribures);
           } else {
                ArrayList<String> tobeReconciledAttributes = new ArrayList<String>(0);
                tobeReconciledAttributes.addAll(attributes);
                newReconUnitData.setReconQueryAttributes(tobeReconciledAttributes);
           }


           System.out.println("Checking if an existing reconciliation unit " +
                              " exists with the same schedule information.\n");
           Collection reconUnits = reconManager.getReconUnits(serviceMO);
           if(!reconUnits.isEmpty()) {
            Iterator it = reconUnits.iterator();
            while(it.hasNext()) {
                ReconUnitData reconUnitData = (ReconUnitData)it.next();
                if (reconUnitData.toString().equals(
                                           newReconUnitData.toString())) {
                    System.out.println("Reconciliation unit with the same " +
                    "schedule already exists, replacing the same with new " + 
                                                                "settings..");
                    it.remove();
                }
            }
           }
           reconUnits.add(newReconUnitData);

           System.out.println("Adding the new recon unit..\n");
           reconManager.setReconUnits(serviceMO, reconUnits);
           System.out.println("Submitting request to run recon");
           Collection existingReconUnits = reconManager.getReconUnits(serviceMO);
           for (Iterator itr = existingReconUnits.iterator(); itr.hasNext();)
           {
                ReconUnitData reconUnitData = (ReconUnitData)itr.next();
                if (reconUnitData.toString().
                        equals(newReconUnitData.toString())) {
                    newReconUnitData = reconUnitData;
                }
            }
           String reconcallback = (String) arguments.get (RECON_CALLBACK);
           Request request;
           if(Boolean.valueOf(reconcallback).booleanValue()) {
               ReconCompleteCallback reconCompleteCallback = 
                                            new ReconCompleteCallback();
               request = reconManager.runRecon(serviceMO, 
                                newReconUnitData, reconCompleteCallback);
           } else {
               request = reconManager.runRecon(serviceMO, newReconUnitData);
           }
           System.out.println("Run recon submitted successfully\n");
           System.out.println("Process ID = " + request.getID());
       } catch (RemoteException e) {
           e.printStackTrace();
       } catch (LoginException e) {
           e.printStackTrace();
       } catch (SchemaViolationException e) {
           e.printStackTrace();
       } catch (AuthorizationException e) {
           e.printStackTrace();
       } catch (ApplicationException e) {
           e.printStackTrace();
       } catch (NumberFormatException e) {
           System.out.println("\nInvalid number format, enter valid numbers\n");
       } catch (AppProcessingException e) {
           e.printStackTrace();
       } catch (RemoteServicesException e) {
           e.printStackTrace();
       } catch (Exception e) {
           e.printStackTrace();
       

       } finally {
            if(platform != null) {
                platform.close();
                platform = null;
            }
       }
   }

   /**
    * Parses the argument list from the command-line
    */
   public static Hashtable parseArgs(String[] args) {
       Hashtable<String,Object> arguments = new Hashtable<String,Object>();
       String argumentList = "";
       for (int i=0;i<args.length;i++) {
           argumentList += args[i];
       }
       try {
           StringTokenizer tokenizer =
               new StringTokenizer(argumentList, "-");
           while (tokenizer.hasMoreTokens()) {
               String token = (String)tokenizer.nextToken();
               int delim = token.indexOf("?");
               String name = token.substring(0, delim);
               String value = token.substring(delim+1, token.length());
               if (arguments.get(name) != null) {
                   // arg name used previous
                   Object vals = arguments.get(name);
                   if (vals instanceof String) {
                       // convert to String[]
                       Vector<String> values = new Vector<String>(1);
                       values.add((String)vals);
                       values.add(value);
                       arguments.put(name, values);
                   } else if (vals instanceof Vector) {
                       // add new element to String[]
                       Vector values = (Vector)vals;
                       values.add(value);
                       arguments.put(name, vals);
                   }
               } else {
                   arguments.put(name, value);
               }
           }
           if (!checkArguments(arguments))
               System.exit(0);
       } catch (Exception e) {
           e.printStackTrace();
           printUsage("Invalid command syntax");
           System.exit(0);
       }
       return arguments;
   }

   public static boolean checkArguments(Hashtable arguments) {
       if (!arguments.containsKey(SERVICE_PROFILE))
           return printUsage("No Service profile specified");
       if (!arguments.containsKey(SERVICE_FILTER))
           return printUsage("No servicefilter specified");
       if (!arguments.containsKey(NAME))
           return printUsage("Name not specified");
       if (!arguments.containsKey(MONTH))
           return printUsage("Month not specified");
       if (!(arguments.containsKey(DAY) || arguments.containsKey(DAY_OF_WEEK)))
           return printUsage("Day or Day of week not specified");
       if (!arguments.containsKey(HOUR))
           return printUsage("Hour not specified");
       if (!arguments.containsKey(MINUTE))
           return printUsage("Minute not specified");
       return true;
   }

   public static boolean printUsage(String msg) {
       StringBuffer usage = new StringBuffer();
       usage.append("\nrunRecon: " + msg + "\n");
       usage.append("usage: runRecon -[argument] ? \"[value]\"\n");
       usage.append("\n");
       usage.append("-serviceprofile      Profile Name of the Service");
       usage.append(" (e.g., WinLocalProfile)\n");
       usage.append("-servicefilter       Ldap Filter to search for a Service instance\n");
       usage.append("-name                name of recon.\n");
       usage.append("-month               month of recon. Valid values 1-12, -1 indicates Monthly, \n");
       usage.append("                     0 to ignore\n");
       usage.append("-day                 day of recon. Valid values 1-31, -1 indicates Daily, \n");
       usage.append("                     0 to ignore.\n");
       usage.append("-dayofweek           day of week. Valid values 1-7.(1 = Sunday, 2 = Monday,..),\n");
       usage.append("                     0 to ignore\n");
       usage.append("\nNOTE: Either day or dayofweek is mandatory. \n");
       usage.append("If specified both, then dayofweek will be ignored\n\n");
       usage.append("-hour                hour of recon. Valid values 0-23, -1 indicates Hourly.\n");
       usage.append("-minute              minute of recon. Valid values 0-59.\n\n");
       usage.append("-[description]       name of description.\n");
       usage.append("-[reconfiler]        Ldap filter to reconcile accounts.\n");
       usage.append("-[excludeattributes] Attribute(s) separated by comma");
       usage.append(" to be excluded during \n                     reconciliation.\n");
       usage.append("-[lockservice]       Specifies if the Service needs to be locked during recon,\n");
       usage.append("                     default value = \"true\"\n");
       usage.append("-[maxduration]       Maximum duration(in minutes) for recon process to run.\n");
       usage.append("                     default value 600 minutes.\n");
       usage.append("-[reconcallback]     If the recon completion details needs to be logged in \n");
       usage.append("                     SystemOut.log. Default value \"false\"\n\n");
       usage.append("Arguments enclosed in [] are optional\n\n");
       usage.append("Example: runRecon -serviceprofile?WinLocalProfile -servicefilter?");
       usage.append("\"(erservicename=WinLocal)\" -reconfilter?\"(eruid=a*)\"");
       usage.append(" -name?\"ADServiceRecon\" -description?\"ADService recon unit\"");
       usage.append(" -excludeattributes?\"erntpasswordneverexpires,description\"");
       usage.append(" -maxduration?10 -month?10 -day?26 -hour?10 -minute?10 -reconcallback?\"true\"");
       usage.append("\n\nNOTE: If negative value -1 needs to be entered for month,day or hour, then \n");
       usage.append("change the StringTokenizer to '+' instead of '-' in function \n");
       usage.append("RunRecon.parseArgs() to identify the arguments prefixed with '+'");
       System.out.println(usage);
       return false;
   }

}